import re
import random

class Bot:
    def __init__(self, nickname):
        """Initialize a bot object

        Args:
            nickname: nickname of the bot on irc
        """
        self.nickname = nickname
        self.channel = None
        self.port = None
        self.server = None
        self.reactions = {}
        self.pings = []
        self.ping_match = re.compile('^(<.+> )?\@?{name}'.format(name=nickname))

    def add_reaction(self, match, reaction):
        """Add a reaction to the bot.

        Args:
            match: The string which, if matched will trigger the answer.
            reaction: The string which will be sent.
        """
        context = {
            'server': self.server,
            'channel': self.channel,
            'name': self.nickname,
        }
        self.reactions[re.compile(match.format(**context))] = reaction

    def add_ping(self, reaction):
        """Add a reaction to a ping"""

        self.pings.append(reaction)

    def get_reaction(self, user, channel, message):
        """Get a reaction to a message.

        Args:
            user: The user who sent the message.
            channel: The channel on which the bot speak.
            message: The message to which the bot has to react.

        Returns:
            Every matched reactions.
        """
        username = user.split('!')[0]

        context = {
            'server': self.server,
            'channel': channel,
            'name': self.nickname,
            'user': username,
            'message': message
        }
        result = []
        for m in self.reactions.keys():
            if m.search(message):
                result.append(self.reactions[m].format(**context))

        if not result and self.ping_match.search(message):
            sentence = random.choice(self.pings).format(**context)
            result.append(' : '.join([username, sentence]))

        return result

